// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

//#pragma warning (disable : 4005)

#include "C:\\Program Files\\Csound6_x64\\include\\csound\\csound.hpp"
#include "CoreMinimal.h"
#include "Components/SynthComponent.h"
#include "DSP/Osc.h"
#include "SimpleCsoundTest.generated.h"


// ========================================================================
// USimpleCsoundTest
// Synth component class which implements USynthComponent
// This is a simple hello-world type example which generates a sine-wave
// tone using a DSP oscillator class and implements a single function to set
// the frequency. To enable example:
// 1. Ensure "SignalProcessing" is added to project's .Build.cs in PrivateDependencyModuleNames
// 2. Enable macro below that includes code utilizing SignalProcessing Oscilator
// ========================================================================

#define SYNTHCOMPONENT_EX_OSCILATOR_ENABLED 0



UCLASS(ClassGroup = Synth, meta = (BlueprintSpawnableComponent))
class CSOUNDTEST_API USimpleCsoundTest : public USynthComponent
{
	GENERATED_BODY()
	
	// Called when synth is created
	virtual bool Init(int32& SampleRate) override;

	// Called to generate more audio
	virtual int32 OnGenerateAudio(float* OutAudio, int32 NumSamples) override;

	// Sets the oscillator's frequency
	UFUNCTION(BlueprintCallable, Category = "Synth|Components|Audio")
	void SetFrequency(const float FrequencyHz = 440.0f);

	
	std::unique_ptr<Csound> csound;
	MYFLT* CSspin, * CSspout;
	int ksmpsIndex = 0;
	int csCompileResult = -1;
protected:
#if SYNTHCOMPONENT_EX_OSCILATOR_ENABLED
	// A simple oscillator class. Can also generate Saw/Square/Tri/Noise.
	Audio::FOsc Osc;
#endif // SYNTHCOMPONENT_EX_OSCILATOR_ENABLED
};